function [data]=ch(data,varargin)
%CH    change SAC header
%    Change header variables for SAC data
%
%    Values should either be in vector form with 
%    a length that matches the number of files or
%    be scalar in which case the value is replicated 
%    for each file.  If a cell vector is given, it
%    must have only one value per cell.
%
%    Usage: new_data=ch(data,field1,value1,field2,value2,...)
%
%    Examples:
%
%    data=ch(data,'DELTA',dt);
%    data=ch(data,'STLA',lat,'STLO',lon)
%    data=ch(data,'KT0','sSKS');
%
%    by Garrett Euler (10/2007)  ggeuler@wustl.edu
%
%    See also:  lh, gh, rsac, bsac, wsac 

% do nothing on no input
if(nargin<=1)
    return
end

% check for unpaired fields
if (nargin>1 && mod(nargin-1,2))
  error('Unpaired Field/Value!')
end

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head'))
    error('sac data structure does not have proper fields')
end

% header names and positions
[numheader,enumheader,lgcheader,chrheader,enumid,enumval]=sachp();

% number of files
n=length(data);

% loop over field/value pairs
for i=1:2:(nargin-1)
    
    % force into cell array
    if(isnumeric(varargin{i+1}))
        varargin{i+1}=num2cell(varargin{i+1});
    elseif(ischar(varargin{i+1}))
        varargin{i+1}=cellstr(varargin{i+1});
    end
    
    % check value is vector
    if(~isvector(varargin{i+1}))
        error('value is not a vector')
    end
    
    % check length (replicate if scalar)
    if(length(varargin{i+1})==1)
        varargin{i+1}=repmat(varargin{i+1},n,1);
    elseif(length(varargin{i+1})~=n)
        error('value vector not correct size')
    end
    
    % force column vector
    varargin{i+1}=varargin{i+1}(:);
    
    % force lowercase
    field=lower(varargin{i});
    
    % loop over files
    for j=1:n
        % check that field exists and write
        if(isfield(numheader,field))
            % float
            if(numheader.(field)<71)
                data(j).head(numheader.(field))=varargin{i+1}{j};
            % integer
            else
                data(j).head(numheader.(field))=round(varargin{i+1}{j});
            end
        elseif(isfield(enumheader,field))
            % identifier
            if(ischar(varargin{i+1}{j}))
                if(isfield(enumval,varargin{i+1}{j}))
                    data(j).head(enumheader.(field))=enumval.(varargin{i+1}{j});
                else
                    disp(sprintf('WARNING: %s IS NOT A KNOWN VALUE - NOT CHANGING!',varargin{i+1}{j}))
                end
            % explicit position
            elseif(isnumeric(varargin{i+1}{j}))
                data(j).head(enumheader.(field))=round(varargin{i+1}{j});
            end
        elseif(isfield(lgcheader,field))
            % 0 or 'f...'
            if(varargin{i+1}{j}(1)==0 || varargin{i+1}{j}(1)==102)
                data(j).head(lgcheader.(field))=0;
            % 1 or 't...'
            elseif(varargin{i+1}{j}(1)==1 || varargin{i+1}{j}(1)==116)
                data(j).head(lgcheader.(field))=1;
            else
                if(ischar(varargin{i+1}{j}))
                    disp(sprintf('WARNING: %s IS NOT A VALID LOGIC VALUE - NOT CHANGING!',varargin{i+1}{j}))
                else
                    disp(sprintf('WARNING: %d IS NOT A VALID LOGIC VALUE - NOT CHANGING!',varargin{i+1}{j}))
                end
            end
        elseif(isfield(chrheader,field))
            k=chrheader.(field); m=k(2)-k(1)+1; l=length(varargin{i+1}{j});
            if (l>m)
                data(j).head(k(1):k(2))=varargin{i+1}{j}(1:m);
            else
                data(j).head(k(1):k(2))=[varargin{i+1}{j} ones(1,m-l)*32];
            end
        else
            disp(sprintf('WARNING: %s IS NOT A KNOWN FIELD!',field))
        end
    end
end

end
