function []=lh(data,varargin)
%LH    list SAC header
%    List SAC header variables for SAC data 
%    similar to a SAC lh display.
%    
%    Usage:    lh(sac_struct,field1,field2,...)
%
%    Examples:
%
%    lh(data)          % lists all header variables
%    lh(data,'dEltA');
%    lh(data,'StLA','stLo')
%
%    by Garrett Euler (10/2007)  ggeuler@wustl.edu
%
%    See also:  ch, gh, rsac, bsac, wsac
%

% do nothing on no input
if(nargin<1)
    return
end

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head'))
    error('sac data structure does not have proper fields')
end

% header names and positions
[numheader,enumheader,lgcheader,chrheader,enumid,enumval,enumdesc]=sachp();

% check for list all
if (nargin==1)
    varargin=[fieldnames(numheader); fieldnames(enumheader);
        fieldnames(lgcheader); fieldnames(chrheader)];
elseif(nargin==2)
    if(strcmpi(varargin{1},'t'))
        varargin={'t0' 't1' 't2' 't3' 't4' 't5' 't6' 't7' 't8' 't9'};
    elseif(strcmpi(varargin{1},'kt'))
        varargin={'kt0' 'kt1' 'kt2' 'kt3' 'kt4' 'kt5' 'kt6' 'kt7' 'kt8' 'kt9'};
    elseif(strcmpi(varargin{1},'resp'))
        varargin={'resp0' 'resp1' 'resp2' 'resp3' 'resp4' 'resp5' 'resp6' 'resp7' 'resp8' 'resp9'};
    elseif(strcmpi(varargin{1},'user'))
        varargin={'user0' 'user1' 'user2' 'user3' 'user4' 'user5' 'user6' 'user7' 'user8' 'user9'};
    elseif(strcmpi(varargin{1},'kuser'))
        varargin={'kuser0' 'kuser1' 'kuser2'};
    end
end

% default character arrays
c8=[45 49 50 51 52 53 32 32];
c16=[45 49 50 51 52 53 32 32 32 32 32 32 32 32 32 32];

% loop over files
for i=1:length(data)
    % print filename
    disp(' ')
    disp(sprintf(' FILE: %s - %d',data(i).name,i))
    disp('------------------------------------')
    disp(' ')
    
    % loop over fields
    for j=1:length(varargin)
        % force lowercase
        field=lower(varargin{j});
        
        % format by type
        if(isfield(numheader,field))
            if(data(i).head(numheader.(field))==-12345)
                disp(sprintf('%-12s = UNDEFINED (%d)',upper(field),data(i).head(numheader.(field))));
            elseif(numheader.(field)<71)
                disp(sprintf('%-12s = %0.8g',upper(field),data(i).head(numheader.(field))));
            else
                disp(sprintf('%-12s = %d',upper(field),round(data(i).head(numheader.(field)))));
            end
        elseif(isfield(enumheader,field))
            ival=round(data(i).head(enumheader.(field)));
            if(ival>=0 && ival<length(enumdesc))
                disp(sprintf('%-12s = %s',upper(field),enumdesc{ival+1}));
            elseif(ival==-12345)
                disp(sprintf('%-12s = UNDEFINED (%d)',upper(field),ival));
            else
                disp(sprintf('%-12s = UNKNOWN (%d)',upper(field),ival));
            end
        elseif(isfield(lgcheader,field))
            if(data(i).head(lgcheader.(field))==0)
                disp(sprintf('%-12s = FALSE',upper(field)));
            elseif(data(i).head(lgcheader.(field))==1)
                disp(sprintf('%-12s = TRUE',upper(field)));  
            elseif(data(i).head(lgcheader.(field))==-12345)
                disp(sprintf('%-12s = UNDEFINED (%d)',upper(field),data(i).head(lgcheader.(field))));
            else
                disp(sprintf('%-12s = INVALID (%f)',upper(field),data(i).head(lgcheader.(field))));
            end
        elseif(isfield(chrheader,field))
            if(isequal(c8,data(i).head(chrheader.(field)(1):chrheader.(field)(2))') || ...
                    isequal(c16,data(i).head(chrheader.(field)(1):chrheader.(field)(2))'))
                disp(sprintf('%-12s = UNDEFINED (%s)',upper(field),...
                    char(data(i).head(chrheader.(field)(1):chrheader.(field)(2)))));
            else
                disp(sprintf('%-12s = %s',upper(field),char(data(i).head(chrheader.(field)(1):chrheader.(field)(2)))));
            end
        end
    end
end

end
