function recsec(data,xlimits,ylimits,scale) 
%RECSEC   plot distance sorted record section of SAC files
%
%     plot SAC seismograms read into matlab with rsac.m 
%      according to distance from event
% 
%     usage:  recsec(data,xlim,ylim,scale)
%
%     where 'data' represents a sac seismogram structure created
%     with rsac or bsac
%
%     xlim and ylim are vectors with the form [time1 time2]
%     and [deg1 deg2] that indicate the range limits for the
%     plot.  These are optional and will scale to fit the
%     data by default.
%
%     scale is optional.  Defaults to normalizing all traces to
%      1 degree max.  Indicates the max amplitude of traces on
%      yaxis (which is degrees).
%
%     Examples:
%
%     To plot the first 4 seismograms in data without windowing
%
%     recsec(data(1:4)) 
%
%     To plot the 5th and 8th seismograms in data with time window
%
%     recsec(data([5 8]),[0 300]) 
%
%     To plot all traces with no time contraint and with an amp of 5deg
%
%     recsec(data,[],[],5)
%
%     by Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also:  p1, p2

% return if no input
if(nargin<1)
    return
end

% check number of inputs
error(nargchk(1,4,nargin))

% initialize plot
figure;
whitebg('k')
set(gcf,'Name','RECSEC -- SAC Seismogram Plotting Utility', ...
    'NumberTitle','off','color','k', ...
    'Pointer','crosshair','PaperOrientation','landscape', ...
    'PaperPosition',[.5 2 10 4.5],'PaperType','usletter');
set(gca,'FontName','times','FontWeight','light', ...
    'xcolor','w','ycolor','w', ...
	'TickDir','out','FontSize',8);
xlabel('Time (sec)')
ylabel('Distance (\circ)')
grid on
hold on

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head') || ~isfield(data,'amp'))
    error('sac data structure does not have proper fields')
end

% get header info
[b,npts,delta,leven,iftype,lcalda,gcarc]=...
    gh(data,'b','npts','delta','leven','iftype','lcalda','gcarc');

% default scale
if(nargin<4)
    scale=1;
end

% loop through each file
for i=1:length(data)
    % check if right filetype (if not skip)
    if(iftype(i)~=1 && iftype(i)~=4)
        continue;
    end
    
    % check gcarc is set
    if(~lcalda(i) || gcarc(i)==-12345)
        continue;
    end
    
    % get max
    amax=max(abs(data(i).amp));
    
    % check if even or uneven timing
    if(leven(i))
        time=b(i)+(0:npts(i)-1)'*delta(i);
    else
        time=data(i).time;
    end
    
    % plot series
    plot(time,gcarc(i)+data(i).amp/amax*scale,'color','w')
end

% limit time
if(nargin>1 && ~isempty(xlimits))
    xlim(xlimits);
end

% limit distance
if(nargin>2 && ~isempty(ylimits))
    ylim(ylimits);
end

hold off

end
