function [data,fs]=sacfilter(data,type,style,limits,order,passes,ripple)
%SACFILTER    Filter SAC data
%       Filters SAC data using the set of supplied parameters to
%       build a IIR filter and implement it.
%
%   Usage: [data,fs]=sacfilter(data,style,type,limits,order,passes,ripple)
%           or [data]=sacfilter(data,fs,passes)
%
%   Type:   'butter', 'cheby1', 'cheby2' or 'ellip'
%   Style:  'high', 'low', 'stop' or 'bandpass'
%   Limits: Array of pass corner(s) and optionally stop corner(s).
%           If stop corner(s) are given then order is ignored and
%           a filter is created using the corners and ripple 
%           values to decide the necessary order.  If no stop
%           corner(s) are given and automatic order calculation
%           is needed, default stop corner(s) are supplied. These
%           are defined in the 'bandpass' and 'stop' case as 1/4 
%           the bandwidth (distance between pass corners) out from
%           the pass corners unless this puts the limit outside the 
%           bandwidth of the signal (0Hz to nyquist) in which case
%           the stop limit is adjusted to 3/4 the width from the 
%           pass corner to the nyquist/0Hz.  The low/high pass
%           case defaults to 1/10 the total bandwidth of the signal
%           above/below the pass corner or 3/4 the width from the 
%           pass corner to the nyquist/0Hz when the prior exceeds
%           the 0Hz/nyquist.  Lowpass or highpass require 1 (pass
%           corner or 2 (both the pass and stop corner) corners
%           as input while bandpass or bandreject filters require
%           2 or 4 corners.  Units in Hz.
%           Special Notes: For filter type 'cheby2' the default
%                          corners are taken to be the stop corner(s)
%                          so all the defaults and methods of obtaining
%                          them are reversed when pass corners are
%                          not also supplied.  For 'butter' the pass
%                          corner(s) correspond to 3dB attenuation
%                          if a custom filter is not created.
%   Order:  Optional.  Not defining or setting to 0 turns automatic
%           filter order calculations on. If no stop corner(s) are 
%           supplied and order is set to 0 (zero), then the order 
%           is found using pass corner(s) and default stop corner(s).
%   Passes: Optional.  Defaults to 1.  Only accepts 1 or 2.  Two
%           passes are convenient to do zero-phase filtering.
%           Note that 2 passes make the signal acausal and should
%           not be done in conjunction with phase picking.
%   Ripple: Optional.  1 or 2 element array of passband ripple 
%           and/or stopband attenuation.  Defaults are 1dB for
%           the passband ripple and 50dB for the stopband 
%           attenuation.  Both values are used when automatically
%           determining the filter order.  Automatic filter 
%           order calculation will be done if both values are
%           supplied unless the filter is elliptic. For elliptic
%           and cheby1 filtering if one value is given it is taken 
%           as the passband ripple.  For cheby2 and butter it is 
%           taken as the stopband attenuation.  Units in dB.
%   fs:     Optional output/input filter structure.  Useful
%           when performing the same filter on multiple files or in
%           multiple sessions (such as when performing a backwards
%           filter pass).  Note that the filter is tied to the 
%           samplerate (the limits are relative to the nyquist).
%           Utilize the fvtool to analyze the filter.
%
%   Examples: 
%
%           4th order lowpass butter filter with a corner of 10s
%        [data]=sacfilter(data,'low','butter',1/10,4)
%
%           bandpass elliptic filter with corners of 15s and 12s using
%           default stop corners and ripples to determine filter order
%        [data]=sacfilter(data,'bandpass','ellip',[1/15 1/12])
%
%           2 pass bandpass elliptic filter with stop corners 
%           defined forcing order calc. 8th order is ignored but the
%           supplied 3dB passband ripple is included in the order calc.
%    [data]=sacfilter(data,'bandpass','ellip',[1/22 1/20 1/15 1/14],8,2,3)
%
%
%   Notes: For narrow band filters especially at lower Hz consider 
%          decimating the signal prior to filtering to aid in the
%          stability of the filter design which uses a finite spacing
%          in the frequency domain for making the filter and if the
%          band is too tight then the corners and magnitudes of the
%          filter will be off.  Try to keep the width above a few 
%          percent of the total bandwidth of the signal.  Try to 
%          shoot for the filter limits to lie between 0.1 and 0.3
%          in the nyquist-normalized frequency domain.  Length of
%          the trace does not affect filter design, but it does affect
%          filter implementation.  
%
%          Implements initial condition determination to limit edge
%          effects caused by lack of mean and trend removal and 
%          non-tapered series.
%
%    by Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also: sacinterp, sacsample, deci, stretch
%

% check number of inputs
error(nargchk(2,7,nargin))

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head') || ~isfield(data,'amp'))
    error('sac data structure does not have proper fields')
elseif(isfield(data,'time'))
    error('function does not work with unevenly sampled data')
end

% number of seismograms
nseis=length(data);
    
% filter pre-made option
if(nargin<4 && isstruct(style))
    % reassign variables (to avoid confusion)
    fs=type;
    if(nargin==3)
        passes=style;
    else
        passes=1;
    end
    clear type style limits order ripple
    
    % implement
    for i=1:nseis
        data(i)=sacimpfilt(data(i),fs);
    end

    % 2 pass section
    if(passes==2)
        for i=1:nseis
            data(i).amp=data(i).amp(end:-1:1);
            data(i)=sacimpfilt(data(i),fs);
            data(i).amp=data(i).amp(end:-1:1);
        end
    end
    
% normal case (filter design and implement)
elseif(strcmpi(type,'butter') || strcmpi(type,'cheby1')...
        || strcmpi(type,'cheby2') || strcmpi(type,'ellip'))
    % pull some header info
    [delta]=gh(data,'delta');
    
    % make delta groups
    [sort_delta,indices]=sort(delta);
    ng=1; b(ng)=1; e(ng)=1; grp_delta(ng)=sort_delta(1);
    for i=2:nseis
        if(sort_delta(i)~=sort_delta(b(ng)))
            e(ng)=i-1;
            ng=ng+1;
            b(ng)=i;
            grp_delta(ng)=sort_delta(i);
        end
    end
    e(ng)=nseis;
    
    % build filter for each group
    for i=1:ng
        % sort limits
        nyq=1/(2*grp_delta(i));
        limits=sort(limits(:)/nyq,1)';
        nlim=length(limits);
        
        % find if above nyquist
        if(~isempty(find(limits>=1,1)))
            error('frequency limits at/above nyquist')
        elseif(~isempty(find(limits<=0,1)))
            error('frequency limits at/below zero')
        end
        
        % dangerously close warnings
        if(~isempty(find(limits>0.85,1)))
            warning('FreqLimit:NyqClose','Dangerously close to the nyquist')
        end
        if(~isempty(find(limits<0.01,1)))
            warning('FreqLimit:ZeroClose','Dangerous low frequency corner')
        end
        
        % set find order off by default
        auto=0;
        
        % setup limits by type (and check type)
        if(strcmpi(style,'low') || strcmpi(style,'high'))
            % check number of limits
            if(nlim~=1 && nlim~=2)
                error('incorrect number of filter limits defined')
            end
            
            % set up limits
            if(strcmpi(style,'low'))
                if(nlim==1)
                    if(~strcmpi(type,'cheby2'))
                        slim=limits+0.1;
                        
                        % check
                        if(slim>1)
                            slim=limits+0.75*(1-limits);
                        end
                    else
                        slim=limits;
                        limits=limits-0.1;
                        
                        % check
                        if(limits<0)
                            limits=0.25*slim;
                        end
                    end
                else
                    slim=limits(2);
                    limits=limits(1);
                    auto=1;
                end
            elseif(strcmpi(style,'high'))
                if(nlim==1)
                    if(~strcmpi(type,'cheby2'))
                        slim=limits-0.1;
                        
                        % check
                        if(slim<0)
                            slim=0.25*limits;
                        end
                    else
                        slim=limits;
                        limits=limits+0.1;
                        
                        % check
                        if(limits>1)
                            limits=slim+0.75*(1-slim);
                        end
                    end
                else
                    slim=limits(1);
                    limits=limits(2);
                    auto=1;
                end
            end
        elseif(strcmpi(style,'stop') || strcmpi(style,'bandpass'))
            % check number of limits
            if(nlim~=2 && nlim~=4)
                error('incorrect number of filter limits defined')
            end
            
            % set up limits
            if(nlim==2)
                if(~strcmpi(type,'cheby2'))
                    slim(1)=limits(1)-0.25*(limits(2)-limits(1));
                    slim(2)=limits(2)+0.25*(limits(2)-limits(1));
                    
                    % check
                    if(slim(1)<0)
                        slim(1)=limits(1)-0.75*limits(1);
                    end
                    if(slim(2)>1)
                        slim(2)=limits(2)+0.75*(1-limits(2));
                    end
                else
                    slim=limits;
                    limits(1)=limits(1)+1/6*(limits(2)-limits(1));
                    limits(2)=limits(2)-1/6*(limits(2)-limits(1));
                end
            else
                slim=[limits(1) limits(4)];
                limits=[limits(2) limits(3)];
                auto=1;
            end
        else
            error('unknown filter style')
        end
        
        % check order
        if(nargin==4 || isempty(order) || order==0)
            auto=1;
        end
        
        % check passes
        if(nargin<6)
            passes=1;
        else
            if(passes~=1 && passes~=2)
                error('1 or 2 passes only')
            end
        end
        
        % check ripple
        rp=1; rs=50;
        if(nargin==7)
            ripple=ripple(:)';
            nr=length(ripple);
            
            % check for negative
            if(ripple(ripple<=0)<=0)
                error('negative or zero ripple not allowed')
            end
            
            % assigning ripple parameters
            if(nr==1)
                if(strcmpi(type,'ellip') || strcmpi(type,'cheby1'))
                    rp=ripple;
                elseif(strcmpi(type,'cheby2'))
                    rs=ripple;
                else
                    rs=ripple;
                    auto=1;
                end
            elseif(nr==2)
                rp=ripple(1);
                rs=ripple(2);
                if(~strcmpi(type,'ellip'))
                    auto=1;
                end
            else
                error('too many ripple parameters')
            end
        end
        
        % find filter order if needed
        if(auto==1)
            % determine by type
            if(strcmpi(type,'butter'))
                [order,limits]=buttord(limits,slim,rp,rs);
            elseif(strcmpi(type,'cheby1'))
                [order,limits]=cheb1ord(limits,slim,rp,rs);
            elseif(strcmpi(type,'cheby2'))
                [order,limits]=cheb2ord(limits,slim,rp,rs);
            else
                [order,limits]=ellipord(limits,slim,rp,rs);
            end
        end
        
        % make the filter
        if(strcmpi(type,'butter'))
            [z,p,k]=butter(order,limits);
        elseif(strcmpi(type,'cheby1'))
            [z,p,k]=cheby1(order,rp,limits);
        elseif(strcmpi(type,'cheby2'))
            [z,p,k]=cheby2(order,rs,limits);
        else
            [z,p,k]=ellip(order,rp,rs,limits);
        end
        
        % put into a filter structure
        [sos,g]=zp2sos(z,p,k);
        fs=dfilt.df2tsos(sos,g);
        
        % implement
        for j=b(i):e(i)
            data(indices(j))=sacimpfilt(data(indices(j)),fs);
        end
        
        % 2 pass section
        if(passes==2)
            for j=b(i):e(i)
                data(indices(j)).amp=data(indices(j)).amp(end:-1:1);
                data(indices(j))=sacimpfilt(data(indices(j)),fs);
                data(indices(j)).amp=data(indices(j)).amp(end:-1:1);
            end
        end
    end
else
    error('Unknown filter type')
end

end

function [data]=sacimpfilt(data,fs)
%SACIMPFILT   Implements filter on SAC data
%   Implements a filter design on a SAC formatted
%   array returning the resulting filtered SAC
%   array with appropriate header changes
%

% pull some header info
npts=gh(data,'npts');

% prepend a mirror-flip of the series
% to limit edge effects
data.amp=filter(fs,[2*data.amp(1)-data.amp(npts:-1:2); data.amp]);
data.amp=data.amp(npts:2*npts-1);
%data.amp=filter(fs,data.amp);

% change header values as required
data=ch(data,'depmen',mean(data.amp),...
    'depmin',min(data.amp),...
    'depmax',max(data.amp));

end