function [data]=sactaper(data,type,width,option)
%SACTAPER   Taper SAC data
%
%   Tapers sac data edges with the specified
%   taper type and width (fraction of total
%   signal length)
%
%   Usage:  [data]=sactaper(data,type,width,option)
%
%       TYPE specifies the taper function type like this:
%          @blackmanharris, @hamming, @hann, @gausswin, etc
%
%          all the types can be found under the window
%          command.  Default is @blackmanharris.
%       
%       WIDTH is the ratio of the taper halfwidth to the 
%          whole data length.  Default is 0.05.  Values
%          are allowed from 0 to 0.5.
%       
%       OPTION is an option specific to the taper type.
%          Look at the window command for more details.
%
%
%   Examples:
%
%     Taper data with a gaussian style that
%       tapers a 10th of the data on each end
%       and giving an override option that makes
%       the end of the taper go to 4 stddev value.
%        data=sactaper(data,@gausswin,0.1,4);
%
%     Default tapering - blackman-harris tapering a
%       20th of the data on each end.
%        data=sactaper(data);
%
%
%    by Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also: rmean, rtrend

% check input
error(nargchk(1,4,nargin))

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head') || ~isfield(data,'amp'))
    error('sac data structure does not have proper fields')
elseif(isfield(data,'time'))
    error('function does not work with unevenly sampled data')
end

% defaults
if(nargin==1)
    type=@blackmanharris;
    width=0.05;
elseif(nargin==2)
    width=0.05;
end
if(isempty(type))
    type=@blackmanharris;
end
if(isempty(width))
    width=0.05;
end

% get npts
npts=gh(data,'npts');

% get taper width in points
nwidth=ceil(width*npts);

% work through each file
for i=1:length(data)
    % make taper
    if(nargin==4)
        taperedge=window(type,2*nwidth(i),option);
    else
        taperedge=window(type,2*nwidth(i));
    end
    
    % over taper cheat (only matters if width>1)
    nwidth(i)=min([nwidth(i) npts(i)]);
    
    % apply taper
    data(i).amp(1:nwidth(i))=...
        data(i).amp(1:nwidth(i)).*taperedge(1:nwidth(i));
    data(i).amp(end-nwidth(i)+1:end)=...
        data(i).amp(end-nwidth(i)+1:end).*taperedge(end-nwidth(i)+1:end);
    
    % adjust header
    data(i)=ch(data(i),'depmen',mean(data(i).amp),...
        'depmin',min(data(i).amp),'depmax',max(data(i).amp));
end

end