function [data]=stretch(data,factor)
%STRETCH    Upsample SAC data
%
%       Upsamples SAC data by an integer
%       factor.  Uses matlabs interp command.
%
%   Usage:  [data]=stretch(data,factor)
%
%       factor is an integer specifying the 
%       factor increase in sample rate. Use
%       cascades if desired (not necessary).
%
%   Examples: 
%
%       to double the samplerates
%       [data]=stretch(data,2)
%
%       to cascade to a samplerate 40 times higher
%       [data]=stretch(data,[5 8])
%
%    by Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also: deci, sacsample, sacinterp, sacfilter
%

% check inputs
error(nargchk(2,2,nargin))

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head') || ~isfield(data,'amp'))
    error('sac data structure does not have proper fields')
elseif(isfield(data,'time'))
    error('function does not work with unevenly sampled data')
end

% number of seismograms
nseis=length(data);

% check factor
if(~isvector(factor) || ~isinteger(factor))
    error('factor must be a vector of integer(s)')
end

% number of factors
nf=length(factor);

% overall factor
of=prod(factor);

% header info
delta=gh(data,'delta');

% loop over files
for i=1:nseis
    % loop over factors
    for j=1:nf
        % stretch
        data(i).amp=interp(data(i).amp,factor(j),4,1);
    end
    
    % update header
    data(i)=ch(data(i),'npts',length(data(i).amp),...
        'delta',delta(i)/of,...
        'depmen',mean(data(i).amp),...
        'depmin',min(data(i).amp),...
        'depmax',max(data(i).amp));
end

end