function []=xsacinterp(file_in,file_out,dt,varargin)
%XSACINTERP    Read, Interpolate, and Write SAC binary files.
%    Reads in SAC (seismic analysis code) binary data
%    using 'rsac', interpolate using 'sacinterp',
%    and finally write sac binary with 'wsac'. 
%
%    Intended to be an external script run via 
%    'matlab -r xsacinterp(...)' on the command line.  
%    Can handle multiple files but it will take some
%    trial and error probably to get the inputs right.
%
%    Escape characters will be needed for running from 
%    the command line.
%
%    Usage: xsacinterp('file_in','file_out',delta,begin_time,end_time,method)
%    NOTE: begin_time, end_time, and method are optional and default
%          to the 'b' and 'e' header values and the 'pchip' interpolation
%          method.
%
%    Examples:
%
%     interpolate at 5 sps using b and e header values and overwrite
%    matlab -nosplash -nojvm -r xsacinterp\(\'KATH.R\'\,\'KATH.R\'\,0.2\)\;exit\;  
%
%     interpolate at 1 sps from 300 seconds to e header value
%    matlab -nosplash -nojvm -r xsacinterp\(\'old.sac\'\,\'new.sac\'\,1\,300\)\;exit\;
%
%     interpolate to 5 sps from 900 to 950 seconds using linear interp
%    matlab -nosplash -nojvm -r xsacinterp\(\'KATH.Z\'\,\'Pdiff.Z\'\,0.2\,900\,950\,\'linear\'\)\;exit\;
%
%    by Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also: sacinterp, rsac, wsac, gh

% check number of arguments
error(nargchk(3,6,nargin))

% read in sac file
[data]=rsac(file_in);

% get begin, end, npts, and overwrite flag of data
[b,e,npts,lovrok]=gh(data,'b','e','npts','lovrok');

% check overwrite
if (~lovrok && (isempty(file_out) || strcmp(file_in,file_out)))
    error('Overwrite not permitted: LOVROK set to FALSE!')
end 

% read in overrides
method='pchip';
if (nargin>3 && ~isempty(varargin{1}))
    b=varargin{1};
end
if (nargin>4 && ~isempty(varargin{2}))
    e=varargin{2};
end
if (nargin>5 && ~isempty(varargin{3}))
    method=varargin{3};
end

% interpolate
[data]=sacinterp(data,dt,b,e,method);

% new filenames
if (~isempty(file_out))
    data.name=file_out;
end

% write out
wsac(data);

end